/***************************************************************
* Name:         garage.c
*
* Purpose:      Implement garage related functions for problem 8 of Lab. 10, Computer Programming
*
* Course code:  21022105
* Student name: Marius Joldos
* E-mail:       Marius.Joldos@cs.utcluj.ro
****************************************************************
* Created:      2011-01-03
*
****************************************************************/
#include "garage.h"
static char buf[128]; // buffer for read
static int readField(int len, const char *prompt, const char *errMsg){
    int c;
    char *pBuf;
    memset(buf, '\0', sizeof(buf));
    printf("%s", prompt);
    fgets(buf, sizeof(buf), stdin);
    int readLen = strlen(buf);
    if (readLen >= len) {
        buf[len-1] = '\0'; // truncate
    } else {
        pBuf = strchr(buf, '\n');
        if (!pBuf) {
            fprintf(stderr, "%s\n", errMsg);
            while ('\n' != (c=getchar())); // skip rest of line
        }
        else *pBuf = '\0';
    }
    if (!buf[0]) return 0; // empty line
    return 1; // duplicate
}
/***********************************************************
* FUNCTION NAME: readCarInfo
*
* DESCRIPTION:  reads info for a car record from stdin
*              return is a static location; it is overwritten
*               by next call
* ARGUMENT LIST: none
* RETURN VALUE: CarT* always same location c
*
* CHANGES: -
*************************************************************/
CarT *readCarInfo(CarT *c){
    memset(c, '\0', sizeof(CarT));
    printf("New car info, end with an empty line:\n");
    if (!readField(31, "Brand:", "Brand too long. Truncated")) return NULL;
    strcpy(c->brand, buf);
    if (!readField(61, "Owner:", "Owner too long. Truncated")) return NULL;
    strcpy(c->owner, buf);
    printf("Color code[0..65535]:");
    if (1 != scanf("%hu%*c", &c->color)){
        fprintf(stderr, "Bad color");
        return NULL;
    }
    if (!readField(8, "Number plate:", "Number plate info too long. Truncated")) return NULL;
    strcpy(c->plate, buf);
    return c;
}
static int cmpOwner(const void *a, const void *b){
    return(strcmp(((CarT *)a)->owner, ((CarT *)b)->owner));
}
/***********************************************************
* FUNCTION NAME: sortCars
*
* DESCRIPTION:  sorts a file with qsort. Allfile is read into memory
*
* ARGUMENT LIST:
*   Type     Argument     IO      Description
* -------- -------------  -- --------------------------------
* const char*   fn        I name offile to sort
* RETURN VALUE: void
*
* CHANGES: input file
*************************************************************/
void sortCars(const char *fn){
    struct stat statbuf;
    int fd = open(fn, O_RDWR, S_IWUSR|S_IRUSR);
    if (fd == -1){
        fprintf(stderr, "sortCars: open: ");
        perror(fn);
        abort();
    }
    if (-1 == fstat (fd, &statbuf) ){
        fprintf(stderr, "sortCars: stat: ");
        perror(fn);
        close(fd);
        abort();
    }
    void *buf = calloc(statbuf.st_size, 1);
    if (!buf){
        fprintf(stderr, "sortCars: alloc failure");
        close(fd);
        abort();
    }
    read(fd, buf, statbuf.st_size);
    lseek(fd, 0, SEEK_SET);
    int n = statbuf.st_size/sizeof(CarT);
    qsort(buf, n, sizeof(CarT), cmpOwner);
    write(fd, buf, statbuf.st_size);
    if (buf) free(buf);
    close(fd);
}
/***********************************************************
* FUNCTION NAME: showCars
*
* DESCRIPTION:  shows the contents of a file with cars to
*               stdout
*
* ARGUMENT LIST:
*   Type     Argument     IO      Description
* -------- -------------  -- --------------------------------
* const char*   fn        I  name of file with cars
* RETURN VALUE: void
*
* CHANGES: -
*************************************************************/
void showCars(const char * fn){
    CarT c;
    int fd = open(fn, O_RDONLY);
    if (fd == -1){
        fprintf(stderr, "sortCars: open: ");
        perror(fn);
        abort();
    }
    printf("\nCars in the garage:\n-------------------\n");
    while (sizeof(c) == read(fd, &c, sizeof(c))){
        printf("Brand: %s\n", c.brand);
        printf("Owner: %s\n", c.owner);
        printf("Color code: %hu\n", c.color);
        printf("Number plate: %s\n", c.plate);
    }
    printf("-------------------\n");
    close(fd);
}
